/* eslint-disable @typescript-eslint/no-explicit-any */
import Cors from '@fastify/cors'
import RateLimit from '@fastify/rate-limit'
import { FileStore } from '@tus/file-store'
import { Server } from '@tus/server'

import { isProduction } from '@redwoodjs/api/logger'
import { createServer } from '@redwoodjs/api-server'

import { logger } from 'src/lib/logger'
import { handleTusUpload } from 'src/lib/tus'

enum Theme {
  light = 'light',
  dark = 'dark',
}

;(async () => {
  const { countries } = await import('countries-list')

  if (!Object.keys(countries).includes(process.env.COUNTRY))
    throw new Error(
      'Invalid COUNTRY environment variable, please select a valid ISO-3166-1 alpha-2 country code\n See https://en.wikipedia.org/wiki/ISO_3166-1#Codes'
    )

  if (!(process.env.DEFAULT_THEME.toLowerCase() in Theme))
    throw new Error(
      'Invalid DEFAULT_THEME environment variable, please select either light or dark'
    )

  logger.info(`Portfolio ${process.env.APP_VERSION}`)

  const server = await createServer({
    logger,
    configureApiServer: async (server) => {
      await server.register(Cors, {
        origin: isProduction
          ? process.env.ADDRESS_PROD
          : process.env.ADDRESS_DEV,
        methods: ['GET', 'POST', 'OPTIONS', 'PATCH', 'HEAD'],
        credentials: isProduction ? true : false,
      })

      await server.register(RateLimit, {
        max: Number(process.env.MAX_HTTP_CONNECTIONS_PER_MINUTE),
        timeWindow: `1 minute`,
      })
    },
  })

  const tusServer = new Server({
    path: '/files',
    respectForwardedHeaders: true,
    datastore: new FileStore({
      directory: `./files_${isProduction ? 'prod' : 'dev'}`,
    }),
    onResponseError(_, err) {
      logger.error(err)
      return { status_code: 500, body: 'Internal Server Error' }
    },
  })

  server.addContentTypeParser(
    'application/offset+octet-stream',
    (_request, _payload, done) => done(null)
  )

  server.all('/files', (req, res) => {
    res.hijack()
    handleTusUpload(req, res, tusServer, false)
  })
  server.all('/files/*', (req, res) => {
    res.hijack()
    handleTusUpload(req, res, tusServer, true)
  })

  await server.start()
})()
