import type {
  QueryResolvers,
  MutationResolvers,
  CreateSocialInput,
  UpdateSocialInput,
  Handle,
} from 'types/graphql'

import { ValidationError } from '@redwoodjs/graphql-server'

import { db } from 'src/lib/db'

const urlRegex =
  /^(?:(?:(?:https?|ftp):)?\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})))(?::\d{2,5})?(?:[/?#]\S*)?$/i

const phoneRegex = /^(\+\d{1,2}\s)?\(?\d{3}\)?[\s.-]\d{3}[\s.-]\d{4}$/

const emailRegex =
  /^([a-zA-Z0-9_\-\.]+)@([a-zA-Z0-9_\-]+)(\.[a-zA-Z]{2,5}){1,2}$/

export const socials: QueryResolvers['socials'] = () => db.social.findMany()

export const social: QueryResolvers['social'] = ({ id }) =>
  db.social.findUnique({
    where: { id },
  })

export const deleteSocial: MutationResolvers['deleteSocial'] = ({ id }) =>
  db.social.delete({
    where: { id },
  })

export const createSocial: MutationResolvers['createSocial'] = ({ input }) => {
  validateInput(input)
  return db.social.create({
    data: input,
  })
}

export const updateSocial: MutationResolvers['updateSocial'] = ({
  id,
  input,
}) => {
  validateInput(input)
  return db.social.update({
    data: input,
    where: { id },
  })
}

const validateInput = (input: CreateSocialInput | UpdateSocialInput) => {
  if (!input.name || input.name.trim().length === 0)
    throw new ValidationError('Name is required')
  if (!input.type) throw new ValidationError('Type is required')

  const urlHandles: Handle[] = ['custom', 'gitea', 'forgejo']

  if (urlHandles.includes(input.type) && !urlRegex.test(input.username))
    throw new ValidationError('Invalid URL')
  else if (input.type === 'phone' && !phoneRegex.test(input.username))
    throw new ValidationError('Invalid Phone Number')
  else if (input.type === 'email' && !emailRegex.test(input.username))
    throw new ValidationError('Invalid Email')
  else if (input.username.trim().length === 0)
    throw new ValidationError('Username is required')
}
