import { useState } from 'react'

import { Meta, UploadResult } from '@uppy/core'
import type {
  CreatePortraitMutation,
  CreatePortraitMutationVariables,
  DeletePortraitMutation,
  DeletePortraitMutationVariables,
  FindPortrait,
  FindPortraitVariables,
  Portrait,
} from 'types/graphql'

import { TypedDocumentNode, useMutation } from '@redwoodjs/web'
import { toast } from '@redwoodjs/web/toast'

import Uploader from 'src/components/Uploader/Uploader'
import { deleteFile, handleBeforeUnload } from 'src/lib/tus'

interface PortraitFormProps {
  portrait?: Portrait
}

export const QUERY: TypedDocumentNode<FindPortrait, FindPortraitVariables> =
  gql`
    query PortraitForm {
      portrait {
        id
        fileId
      }
    }
  `

const DELETE_PORTRAIT_MUTATION: TypedDocumentNode<
  DeletePortraitMutation,
  DeletePortraitMutationVariables
> = gql`
  mutation DeletePortraitMutation {
    deletePortrait {
      id
      fileId
    }
  }
`

const CREATE_PORTRAIT_MUTATION: TypedDocumentNode<
  CreatePortraitMutation,
  CreatePortraitMutationVariables
> = gql`
  mutation CreatePortraitMutation($input: CreatePortraitInput!) {
    createPortrait(input: $input) {
      id
      fileId
    }
  }
`

const PortraitForm = ({ portrait }: PortraitFormProps) => {
  const [fileId, setFileId] = useState<string>(portrait?.fileId)

  const unloadAbortController = new AbortController()

  const [deletePortrait, { loading: deleteLoading }] = useMutation(
    DELETE_PORTRAIT_MUTATION,
    {
      onCompleted: () => {
        toast.success('Portrait deleted')
      },
      onError: (error) => {
        toast.error(error.message)
      },
      refetchQueries: [{ query: QUERY }],
      awaitRefetchQueries: true,
    }
  )

  const [createPortrait, { loading: createLoading }] = useMutation(
    CREATE_PORTRAIT_MUTATION,
    {
      onCompleted: () => toast.success('Portrait saved'),
      onError: (error) => toast.error(error.message),
      refetchQueries: [{ query: QUERY }],
      awaitRefetchQueries: true,
    }
  )

  const onUploadComplete = (
    result: UploadResult<Meta, Record<string, never>>
  ) => {
    setFileId(result.successful[0]?.uploadURL)
    window.addEventListener(
      'beforeunload',
      (e) => handleBeforeUnload(e, [fileId]),
      {
        once: true,
        signal: unloadAbortController.signal,
      }
    )
  }

  if (portrait?.fileId)
    return (
      <div className="mx-auto w-fit space-y-2">
        <img
          className="aspect-portrait max-w-80 sm:max-w-sm md:max-w-md lg:max-w-lg xl:max-w-xl 2xl:max-w-2xl rounded-xl object-cover"
          src={portrait?.fileId}
          alt={`${process.env.FIRST_NAME} Portrait`}
        />
        <div className="flex justify-center">
          <button
            type="button"
            title="Delete portrait"
            className="btn btn-error btn-sm uppercase"
            onClick={() => {
              if (confirm('Are you sure?')) {
                deleteFile(portrait?.fileId)
                deletePortrait()
                setFileId(null)
              }
            }}
            disabled={deleteLoading}
          >
            Delete
          </button>
        </div>
      </div>
    )
  else
    return (
      <div className="mx-auto max-w-prose space-y-2">
        {!fileId ? (
          <>
            <Uploader
              onComplete={onUploadComplete}
              width="auto"
              height="30rem"
            />
            <p className="text-center">
              High quality, 4:5 aspect ratio image recommended
            </p>
          </>
        ) : (
          <img
            className="aspect-portrait max-w-2xl rounded-xl object-cover"
            src={fileId}
            alt={`${process.env.FIRST_NAME} Portrait`}
          />
        )}
        {fileId && (
          <div className="flex justify-center space-x-2">
            <button
              className={`btn btn-sm ${!fileId && 'btn-disabled'} uppercase`}
              disabled={!fileId || deleteLoading}
              onClick={() => {
                deleteFile(fileId)
                setFileId(null)
                unloadAbortController.abort()
              }}
            >
              Change
            </button>
            <button
              className={`btn btn-primary btn-sm ${
                !fileId && 'btn-disabled'
              } uppercase`}
              disabled={!fileId || createLoading}
              onClick={() => {
                createPortrait({
                  variables: {
                    input: {
                      fileId,
                    },
                  },
                })
                unloadAbortController.abort()
              }}
            >
              Submit
            </button>
          </div>
        )}
      </div>
    )
}

export default PortraitForm
