import { mdiLinkVariant } from '@mdi/js'
import Icon from '@mdi/react'
import { format, isAfter, startOfToday } from 'date-fns'
import parseHtml from 'react-html-parser'
import type { FindProjectById } from 'types/graphql'

import { calculateLuminance } from 'src/lib/color'

interface Props {
  project: NonNullable<FindProjectById['project']>
}

const Project = ({ project }: Props) => {
  return (
    <div className="grid grid-rows-1 grid-cols-1 sm:grid-cols-2">
      <div className="flex flex-col gap-8 p-8">
        <h1 className="text-5xl font-bold font-syne w-fit">{project.title}</h1>
        <div className="flex flex-wrap gap-2 w-fit">
          {isAfter(new Date(project.date), startOfToday()) && (
            <div className="badge badge-lg badge-info whitespace-nowrap">
              planned
            </div>
          )}
          <div className="badge badge-lg badge-ghost whitespace-nowrap">
            {format(project.date, 'PPP')}
          </div>
        </div>
        {project.tags.length > 0 && (
          <div className="flex flex-wrap gap-2 w-fit">
            {project.tags.map((tag, i) => (
              <div
                key={i}
                className="badge whitespace-nowrap"
                style={{
                  backgroundColor: tag.color,
                  color:
                    calculateLuminance(tag.color) > 0.5 ? 'black' : 'white',
                }}
              >
                {tag.tag}
              </div>
            ))}
          </div>
        )}
        {project.description && (
          <article className="prose">{parseHtml(project.description)}</article>
        )}
        {project.links.length > 0 && (
          <>
            <h2 className="font-bold text-3xl w-fit">Links</h2>
            <div className="flex flex-col gap-2 w-fit">
              <ul className="list-none">
                {project.links.map((link, i) => (
                  <li key={i}>
                    <div className="flex gap-2 items-center justify-start">
                      <Icon path={mdiLinkVariant} className="size-4" />
                      <a
                        href={link}
                        target="_blank"
                        rel="noreferrer"
                        className="list-item link link-hover"
                      >
                        {link}
                      </a>
                    </div>
                  </li>
                ))}
              </ul>
            </div>
          </>
        )}
        {project.images.length > 0 && (
          <h2 className="sm:hidden font-bold text-3xl text-center">Images</h2>
        )}
      </div>
      <div className="flex flex-wrap gap-4 pt-8 justify-center h-fit sm:p-8">
        {project.images.length > 0 &&
          project.images.map((image, i) => (
            <a
              href={image}
              target="_blank"
              rel="noreferrer"
              key={i}
              className="rounded-box"
            >
              <img src={image} alt="" className="rounded-xl" />
            </a>
          ))}
      </div>
    </div>
  )
}

export default Project
