import { useState } from 'react'

import { Meta, UploadResult } from '@uppy/core'
import type {
  CreateResumeMutation,
  CreateResumeMutationVariables,
  DeleteResumeMutation,
  DeleteResumeMutationVariables,
  FindResume,
  FindResumeVariables,
  Resume,
} from 'types/graphql'

import { TypedDocumentNode, useMutation } from '@redwoodjs/web'
import { toast } from '@redwoodjs/web/toast'

import PDF from 'src/components/PDF/PDF'
import Uploader from 'src/components/Uploader/Uploader'
import { deleteFile, handleBeforeUnload } from 'src/lib/tus'

interface ResumeFormProps {
  resume?: Resume
}

export const QUERY: TypedDocumentNode<FindResume, FindResumeVariables> = gql`
  query ResumeForm {
    resume {
      id
      fileId
    }
  }
`

const DELETE_RESUME_MUTATION: TypedDocumentNode<
  DeleteResumeMutation,
  DeleteResumeMutationVariables
> = gql`
  mutation DeleteResumeMutation {
    deleteResume {
      id
      fileId
    }
  }
`

const CREATE_RESUME_MUTATION: TypedDocumentNode<
  CreateResumeMutation,
  CreateResumeMutationVariables
> = gql`
  mutation CreateResumeMutation($input: CreateResumeInput!) {
    createResume(input: $input) {
      id
      fileId
    }
  }
`

const ResumeForm = ({ resume }: ResumeFormProps) => {
  const [fileId, setFileId] = useState<string>(resume?.fileId)

  const unloadAbortController = new AbortController()

  const [deleteResume, { loading: deleteLoading }] = useMutation(
    DELETE_RESUME_MUTATION,
    {
      onCompleted: () => {
        toast.success('Resume deleted')
      },
      onError: (error) => {
        toast.error(error.message)
      },
      refetchQueries: [{ query: QUERY }],
      awaitRefetchQueries: true,
    }
  )

  const [createResume, { loading: createLoading }] = useMutation(
    CREATE_RESUME_MUTATION,
    {
      onCompleted: () => toast.success('Resume saved'),
      onError: (error) => toast.error(error.message),
      refetchQueries: [{ query: QUERY }],
      awaitRefetchQueries: true,
    }
  )

  const onUploadComplete = (
    result: UploadResult<Meta, Record<string, never>>
  ) => {
    setFileId(result.successful[0]?.uploadURL)
    window.addEventListener(
      'beforeunload',
      (e) => handleBeforeUnload(e, [fileId]),
      {
        once: true,
        signal: unloadAbortController.signal,
      }
    )
  }

  if (resume?.fileId)
    return (
      <div className="mx-auto w-fit space-y-2">
        <PDF form url={resume?.fileId} />
        <div className="flex justify-center">
          <button
            type="button"
            title="Delete resume"
            className="btn btn-error btn-sm uppercase"
            onClick={() => {
              if (confirm('Are you sure?')) {
                deleteFile(resume?.fileId)
                deleteResume()
                setFileId(null)
              }
            }}
            disabled={deleteLoading}
          >
            Delete
          </button>
        </div>
      </div>
    )
  else
    return (
      <div className="mx-auto max-w-prose space-y-2">
        {!fileId ? (
          <>
            <Uploader
              onComplete={onUploadComplete}
              width="auto"
              height="30rem"
              type="pdf"
            />
          </>
        ) : (
          <PDF form url={fileId} />
        )}
        {fileId && (
          <div className="flex justify-center space-x-2">
            <button
              className={`btn btn-sm ${!fileId && 'btn-disabled'} uppercase`}
              disabled={!fileId || deleteLoading}
              onClick={() => {
                deleteFile(fileId)
                setFileId(null)
                unloadAbortController.abort()
                console.log('aborted')
              }}
            >
              Change
            </button>
            <button
              className={`btn btn-primary btn-sm ${
                !fileId && 'btn-disabled'
              } uppercase`}
              disabled={!fileId || createLoading}
              onClick={() => {
                createResume({
                  variables: {
                    input: {
                      fileId,
                    },
                  },
                })
                unloadAbortController.abort()
                console.log('aborted')
              }}
            >
              Submit
            </button>
          </div>
        )}
      </div>
    )
}

export default ResumeForm
