import { mdiDotsVertical } from '@mdi/js'
import Icon from '@mdi/react'
import type {
  DeleteSocialMutation,
  DeleteSocialMutationVariables,
  FindSocials,
} from 'types/graphql'

import { Link, routes } from '@redwoodjs/router'
import { useMutation } from '@redwoodjs/web'
import type { TypedDocumentNode } from '@redwoodjs/web'
import { toast } from '@redwoodjs/web/toast'

import { QUERY } from 'src/components/Social/SocialsCell'
import { truncate } from 'src/lib/formatters'
import { getLogoComponent, sortOrder } from 'src/lib/handle'

const DELETE_SOCIAL_MUTATION: TypedDocumentNode<
  DeleteSocialMutation,
  DeleteSocialMutationVariables
> = gql`
  mutation DeleteSocialMutation($id: Int!) {
    deleteSocial(id: $id) {
      id
    }
  }
`

const SocialsList = ({ socials }: FindSocials) => {
  const [deleteSocial] = useMutation(DELETE_SOCIAL_MUTATION, {
    onCompleted: () => {
      toast.success('Social deleted')
    },
    onError: (error) => {
      toast.error(error.message)
    },
    refetchQueries: [{ query: QUERY }],
    awaitRefetchQueries: true,
  })

  const onDeleteClick = (
    name: string,
    id: DeleteSocialMutationVariables['id']
  ) => {
    if (confirm(`Are you sure you want to delete ${name}?`))
      deleteSocial({ variables: { id } })
  }

  return (
    <div className="w-full overflow-hidden overflow-x-auto rounded-xl bg-base-100">
      <table className="table">
        <thead className="bg-base-200 font-syne">
          <tr>
            <th className="w-0">Type</th>
            <th className="w-1/2">Name</th>
            <th className="w-1/2">Username</th>
            <th className="w-0">&nbsp;</th>
          </tr>
        </thead>
        <tbody>
          {[...socials]
            .sort(
              (a, b) => sortOrder.indexOf(a.type) - sortOrder.indexOf(b.type)
            )
            .map((social) => {
              const actionButtons = (
                <>
                  <Link
                    to={routes.social({ id: social.id })}
                    title={'Show social ' + social.id + ' detail'}
                    className="btn btn-xs uppercase"
                  >
                    Show
                  </Link>
                  <Link
                    to={routes.editSocial({ id: social.id })}
                    title={'Edit social ' + social.id}
                    className="btn btn-primary btn-xs uppercase"
                  >
                    Edit
                  </Link>
                  <button
                    type="button"
                    title={'Delete social ' + social.id}
                    className="btn btn-error btn-xs uppercase"
                    onClick={() => onDeleteClick(social.name, social.id)}
                  >
                    Delete
                  </button>
                </>
              )

              return (
                <tr key={social.id}>
                  <th>{getLogoComponent(social.type)}</th>
                  <td>{truncate(social.name)}</td>
                  <td>{truncate(social.username)}</td>
                  <td>
                    <nav className="hidden justify-end space-x-2 sm:flex">
                      {actionButtons}
                    </nav>
                    <div className="dropdown dropdown-end flex justify-end sm:hidden">
                      <div
                        tabIndex={0}
                        role="button"
                        className="btn btn-square btn-ghost btn-sm lg:hidden"
                      >
                        <Icon
                          path={mdiDotsVertical}
                          className="text-base-content-100 size-6"
                        />
                      </div>
                      <div
                        // eslint-disable-next-line jsx-a11y/no-noninteractive-tabindex
                        tabIndex={0}
                        className="menu dropdown-content z-10 -mt-1 mr-10 inline rounded-box bg-base-100 shadow-xl"
                      >
                        <nav className="w-46 space-x-2">{actionButtons}</nav>
                      </div>
                    </div>
                  </td>
                </tr>
              )
            })}
        </tbody>
      </table>
    </div>
  )
}

export default SocialsList
