import type {
  EditTagById,
  UpdateTagInput,
  UpdateTagMutationVariables,
} from 'types/graphql'

import { navigate, routes } from '@redwoodjs/router'
import type {
  CellSuccessProps,
  CellFailureProps,
  TypedDocumentNode,
} from '@redwoodjs/web'
import { useMutation } from '@redwoodjs/web'
import { toast } from '@redwoodjs/web/toast'

import CellFailure from 'src/components/Cell/CellFailure/CellFailure'
import CellLoading from 'src/components/Cell/CellLoading/CellLoading'
import TagForm from 'src/components/Tag/TagForm'

export const QUERY: TypedDocumentNode<EditTagById> = gql`
  query EditTagById($id: Int!) {
    tag: tag(id: $id) {
      id
      tag
      color
    }
  }
`

const UPDATE_TAG_MUTATION: TypedDocumentNode<
  EditTagById,
  UpdateTagMutationVariables
> = gql`
  mutation UpdateTagMutation($id: Int!, $input: UpdateTagInput!) {
    updateTag(id: $id, input: $input) {
      id
      tag
      color
    }
  }
`

export const Loading = () => <CellLoading />

export const Failure = ({ error }: CellFailureProps) => (
  <CellFailure error={error} />
)

export const Success = ({ tag }: CellSuccessProps<EditTagById>) => {
  const [updateTag, { loading, error }] = useMutation(UPDATE_TAG_MUTATION, {
    onCompleted: () => {
      toast.success('Tag updated')
      navigate(routes.tags())
    },
    onError: (error) => toast.error(error.message),
  })

  const onSave = (input: UpdateTagInput, id: EditTagById['tag']['id']) =>
    updateTag({ variables: { id, input } })

  return (
    <div className="flex mx-auto max-w-80 sm:max-w-sm md:max-w-md lg:max-w-lg xl:max-w-xl 2xl:max-w-2xl justify-center">
      <div className="overflow-hidden w-full overflow-x-auto rounded-xl bg-base-100">
        <table className="table">
          <thead className="bg-base-200 font-syne">
            <tr>
              <th>Edit Tag {tag?.id}</th>
            </tr>
          </thead>
          <tbody>
            <tr>
              <th>
                <TagForm
                  tag={tag}
                  onSave={onSave}
                  error={error}
                  loading={loading}
                />
              </th>
            </tr>
          </tbody>
        </table>
      </div>
    </div>
  )
}
