import { useEffect, useRef, useState } from 'react'

import { mdiTag } from '@mdi/js'
import Icon from '@mdi/react'
import type { EditTagById, UpdateTagInput } from 'types/graphql'

import type { RWGqlError } from '@redwoodjs/forms'
import { Form, FieldError, Label, TextField, Submit } from '@redwoodjs/forms'

import ColorPicker from 'src/components/ColorPicker/ColorPicker'
import { calculateLuminance } from 'src/lib/color'

type FormTag = NonNullable<EditTagById['tag']>

interface TagFormProps {
  tag?: EditTagById['tag']
  onSave: (data: UpdateTagInput, id?: FormTag['id']) => void
  error: RWGqlError
  loading: boolean
}

const TagForm = (props: TagFormProps) => {
  const [tag, setTag] = useState<string>(props.tag?.tag ?? '')
  const [color, setColor] = useState<string>(props.tag?.color || '#e5e6e6')

  const onSubmit = (data: FormTag) => {
    data.color = color
    props.onSave(data, props?.tag?.id)
  }

  const tagRef = useRef<HTMLInputElement>(null)
  useEffect(() => tagRef.current?.focus(), [])

  return (
    <Form<FormTag>
      onSubmit={onSubmit}
      error={props.error}
      className="space-y-2"
    >
      <Label name="tag" className="form-control w-full">
        <Label
          name="tag"
          className="input input-bordered flex items-center gap-2"
          errorClassName="input input-bordered flex items-center gap-2 input-error"
        >
          <Label
            name="tag"
            className="size-4 opacity-70"
            errorClassName="size-4 text-error"
          >
            <Icon path={mdiTag} />
          </Label>
          <TextField
            name="tag"
            ref={tagRef}
            placeholder="Tag"
            defaultValue={tag}
            className="w-full"
            maxLength={16}
            onChange={(e) => setTag(e.target.value.slice(0, 16))}
            validation={{
              required: {
                value: true,
                message: 'Required',
              },
            }}
          />
        </Label>
        <div className="label">
          <FieldError name="tag" className="text-xs font-semibold text-error" />
        </div>
      </Label>

      <ColorPicker color={color} setColor={setColor} />

      <div className="flex justify-center py-2">
        <div
          className={`badge ${tag.length === 0 && 'hidden'}`}
          style={{
            backgroundColor: color,
            color: calculateLuminance(color) > 0.5 ? 'black' : 'white',
          }}
        >
          {tag}
        </div>
      </div>

      <nav className="my-2 flex justify-center space-x-2">
        <Submit
          disabled={props.loading}
          className="btn btn-primary btn-sm uppercase"
        >
          Save
        </Submit>
      </nav>
    </Form>
  )
}

export default TagForm
