import { mdiDotsVertical } from '@mdi/js'
import Icon from '@mdi/react'
import type {
  DeleteTagMutation,
  DeleteTagMutationVariables,
  FindTags,
} from 'types/graphql'

import { Link, routes } from '@redwoodjs/router'
import { useMutation } from '@redwoodjs/web'
import type { TypedDocumentNode } from '@redwoodjs/web'
import { toast } from '@redwoodjs/web/toast'

import { QUERY } from 'src/components/Tag/TagsCell'
import { calculateLuminance } from 'src/lib/color'
import { truncate } from 'src/lib/formatters'

const DELETE_TAG_MUTATION: TypedDocumentNode<
  DeleteTagMutation,
  DeleteTagMutationVariables
> = gql`
  mutation DeleteTagMutation($id: Int!) {
    deleteTag(id: $id) {
      id
    }
  }
`

const TagsList = ({ tags }: FindTags) => {
  const [deleteTag] = useMutation(DELETE_TAG_MUTATION, {
    onCompleted: () => toast.success('Tag deleted'),
    onError: (error) => toast.error(error.message),
    refetchQueries: [{ query: QUERY }],
    awaitRefetchQueries: true,
  })

  const onDeleteClick = (id: DeleteTagMutationVariables['id']) => {
    if (confirm('Are you sure you want to delete tag ' + id + '?'))
      deleteTag({ variables: { id } })
  }

  return (
    <div className="w-full overflow-hidden overflow-x-auto rounded-xl bg-base-100">
      <table className="table">
        <thead className="bg-base-200 font-syne">
          <tr>
            <th className="w-0">Color</th>
            <th className="w-1/2">Tag</th>
            <th className="w-1/2">Preview</th>
            <th className="w-0">&nbsp;</th>
          </tr>
        </thead>
        <tbody className="overflow-y-scroll">
          {tags.map((tag, i) => {
            const actionButtons = (
              <>
                <Link
                  to={routes.tag({ id: tag.id })}
                  title={'Show tag ' + tag.id + ' detail'}
                  className="btn btn-xs uppercase"
                >
                  Show
                </Link>
                <Link
                  to={routes.editTag({ id: tag.id })}
                  title={'Edit social ' + tag.id}
                  className="btn btn-primary btn-xs uppercase"
                >
                  Edit
                </Link>
                <button
                  type="button"
                  title={'Delete social ' + tag.id}
                  className="btn btn-error btn-xs uppercase"
                  onClick={() => onDeleteClick(tag.id)}
                >
                  Delete
                </button>
              </>
            )

            return (
              <tr key={tag.id}>
                <th>
                  <div
                    key={i}
                    className="tooltip tooltip-right"
                    data-tip={tag.color}
                  >
                    <div
                      className="size-8 rounded-md"
                      style={{ backgroundColor: tag.color }}
                    />
                  </div>
                </th>
                <td>{truncate(tag.tag)}</td>
                <td>
                  <div
                    className="badge whitespace-nowrap"
                    style={{
                      backgroundColor: tag.color,
                      color:
                        calculateLuminance(tag.color) > 0.5 ? 'black' : 'white',
                    }}
                  >
                    {tag.tag}
                  </div>
                </td>
                <td>
                  <nav className="hidden justify-end space-x-2 sm:flex">
                    {actionButtons}
                  </nav>
                  <div className="dropdown dropdown-end flex justify-end sm:hidden">
                    <div
                      tabIndex={0}
                      role="button"
                      className="btn btn-square btn-ghost btn-sm lg:hidden"
                    >
                      <Icon
                        path={mdiDotsVertical}
                        className="text-base-content-100 size-6"
                      />
                    </div>
                    <div
                      // eslint-disable-next-line jsx-a11y/no-noninteractive-tabindex
                      tabIndex={0}
                      className="menu dropdown-content z-10 -mt-1 mr-10 inline rounded-box bg-base-100 shadow-xl"
                    >
                      <nav className="w-46 space-x-2">{actionButtons}</nav>
                    </div>
                  </div>
                </td>
              </tr>
            )
          })}
        </tbody>
      </table>
    </div>
  )
}

export default TagsList
